/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "AMRLevel.H"
#include "AMRLevelGodunov.H"
#include "AMRLevelGodunovFactory.H"

AMRLevelGodunovFactory::AMRLevelGodunovFactory()
{
  setDefaultValues();
}

// "virtual constructor"
AMRLevel* AMRLevelGodunovFactory::new_amrlevel() const
{
  AMRLevelGodunov* amrg_ptr = new AMRLevelGodunov();

  amrg_ptr->CFL(m_cfl);
  amrg_ptr->physIBC(m_phys_ibc_ptr);
  amrg_ptr->refinementThreshold(m_refine_thresh);
  amrg_ptr->tagBufferSize(m_tag_buffer_size);
  amrg_ptr->initialDtMultiplier(m_initial_dt_multiplier);

  return (static_cast <AMRLevel*> (amrg_ptr));
}

AMRLevelGodunovFactory::~AMRLevelGodunovFactory()
{
}

// set physical boundary conditions object
void AMRLevelGodunovFactory::physIBC(RefCountedPtr<PhysIBC> a_phys_ibc_ptr)
{
  m_phys_ibc_ptr = a_phys_ibc_ptr;
}

// set the CFL number
void AMRLevelGodunovFactory::CFL(Real a_cfl)
{
  m_cfl = a_cfl;
}

// set the refinement threshold
void AMRLevelGodunovFactory::refinementThreshold(Real a_refine_thresh)
{
  m_refine_thresh = a_refine_thresh;
}

// set the tag buffer size
void AMRLevelGodunovFactory::tagBufferSize(int a_tag_buffer_size)
{
  m_tag_buffer_size = a_tag_buffer_size;
}

// set the initial dt multiplier
void AMRLevelGodunovFactory::initialDtMultiplier(Real a_initial_dt_multiplier)
{
  m_initial_dt_multiplier = a_initial_dt_multiplier;
}

void AMRLevelGodunovFactory::setDefaultValues()
{
  m_cfl = 0.8;
  m_refine_thresh = 0.2;
  m_phys_ibc_ptr = NULL;
  m_tag_buffer_size = 2;
  m_initial_dt_multiplier = 0.1;
}
