/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef  RAMPIBC_H_
#define  RAMPIBC_H_

#include  <iostream>

#include "LevelData.H"
#include "FArrayBox.H"
#include "Vector.H"
#include "RealVect.H"
#include "AMRIO.H"

#include "PhysIBC.H"
#include "SolidF_F.H"

class RampIBC:public PhysIBC
{
public:
  RampIBC();
  
  RampIBC(const Real& a_gamma,
          const Real& a_alpha,
          const Real& a_ms,
          const Real& a_xcorner);

  ~RampIBC(){};

  void setFortranCommon(const Real& a_gamma,
                        const Real& a_alpha,
                        const Real& a_ms,
                        const Real& a_xcorner);

  void fluxBC(FArrayBox&       a_flux,
              const FArrayBox& a_conState,
              const FArrayBox&       a_divu,
              const FArrayBox&       a_qgdnv,
              const FArrayBox&       a_prim,
              const int&        a_dir,
              const Real&      a_time,
              const Real&      a_dt,
              const Real&      a_dx,
              const Box&       a_DProblem
              ) const;
  
  void fluxBC(FArrayBox&           a_flux,
              const FArrayBox&     a_conState,
              const FArrayBox&           a_artVisc,
              const FArrayBox&           a_qGdnv,
              const FArrayBox&           a_prim,
              const int&            a_dir,
              const Real&          a_time,
              const Real&          a_dt,
              const Real&          a_dx,
              const ProblemDomain& a_DProblem
              ) const;
  
  void setBdrySlopes(FArrayBox&       a_dq,
                     const Box&       a_DProblem,
                     const Real&      a_dx,
                     const int&       a_dir,
                     const FArrayBox& a_prim
                     ) const;
  
  void setBdrySlopes(FArrayBox&       a_dq,
                     const ProblemDomain& a_DProblem,
                     const Real&          a_dx,
                     const int&           a_dir,
                     const FArrayBox&     a_prim) const;
  
  void initialize(LevelData<FArrayBox>& a_conState,
                  const Box&                   a_DProblem,
                  const Real&                  a_dx) const;
  
  void initialize(LevelData<FArrayBox>& a_conState,
                  const ProblemDomain&        a_DProblem,
                  const Real&                  a_dx) const;
  
protected:
  bool m_isFortranCommonSet;
};

#endif
