C     -*- Mode: Fortran; -*-
c-----------------------------------------------------------------
c     Ravi Samtaney
c     Copyright 2002
c     Princeton Plasma Physics Laboratory
c     All Rights Reserved
c-----------------------------------------------------------------
c     $Log: constructlrstates.F,v $
c     Revision 1.1  2002/06/13 22:16:18  samtaney
c     Original unsplit 8-wave Ideal MHD source (includes B filter).
c
c-----------------------------------------------------------------
      subroutine ConstructLRStates(v,vl,vr,dvdx,ilo,ihi,direction,
     &     dtodx)
c     
      implicit none
c     
#include "EosCommon.fh"
c     
c     Incoming arguments
      integer ilo,ihi,direction
      real*8 v(ilo:ihi,8)
      real*8 vl(ilo:ihi,8)
      real*8 vr(ilo:ihi,8)
      real*8 dvdx(ilo:ihi,8)
      real*8 dtodx
c     
c     Local variables
      integer i,k,l,m
      real*8 lambda(8)
      real*8 evl(8,8), evr(8,8)
      real*8 rhoInv
      real*8 Asq
      real*8 axsq
      real*8 csndsq
      real*8 cfast
      real*8 cslow
      real*8 tmp
      real*8 alphaS,alphaF
      real*8 betaY,betaZ
      real*8 betaPerpMagInv
      real*8 Sbx
      real*8 sum1
      real*8 pl(8), pr(8), up, um
c     
      real*8 TwoSqrtInv
      real*8 betaPerpMag
      real*8 tolerance
c     
      TwoSqrtInv=0.70710678D0
      tolerance=1.D-08
c     
      do i=ilo,ihi,1
         do m=1,8,1
            vl(i,m)=v(i,m)
            vr(i,m)=v(i,m)
         enddo
c     vl(i,1:8)=v(i,1:8)+0.5D0*dvdx(i,1:8)
c     vr(i,1:8)=v(i,1:8)-0.5D0*dvdx(i,1:8)
      enddo	
c     
      do i=ilo,ihi,1
c
         do l=1,8,1
            do m=1,8,1
               evl(l,m)=0.D0
               evr(l,m)=0.D0
            enddo
         enddo
c     
         rhoInv=1.D0/v(i,1)
         axsq=v(i,5)**2*rhoInv
         Asq=(v(i,5)**2+v(i,6)**2+v(i,7)**2)*rhoInv
         csndsq=gamma*v(i,8)*rhoInv
         tmp=dsqrt(dmax1(((csndsq+Asq)**2-4.0*csndsq*axsq),0.D0))
         cfast=0.5D0*(csndsq+Asq+tmp)
         cslow=0.5D0*(csndsq+Asq-tmp)
c     
         alphaF=dsqrt(dmax1(csndsq-cslow,0.D0)/(cfast-cslow))
         alphaS=dsqrt(dmax1(cfast-csndsq,0.D0)/(cfast-cslow))
c     
         Sbx=dsign(1.D0,v(i,5))
c     
         betaPerpMag=dsqrt(v(i,6)**2+v(i,7)**2)
c     if(v(i,6).eq.0.D0.and.v(i,7).eq.0.D0) then
         if(dabs(betaPerpMag).lt.tolerance) then
            betaPerpMagInv=1.D0
            betaY=TwoSqrtInv
            betaZ=TwoSqrtInv
         else
c     betaPerpMagInv=1.D0/dsqrt(v(i,6)**2+v(i,7)**2)
            betaPerpMagInv=1.D0/betaPerpMag
            betaY=v(i,6)*betaPerpMagInv
            betaZ=v(i,7)*betaPerpMagInv
         endif
c     
         cfast=dsqrt(cfast)
         cslow=dsqrt(dmax1(cslow,0.D0))
c     
         axsq=dsqrt(dmax1(axsq,0.D0))
c     
         lambda(1)=v(i,2)
         lambda(2)=v(i,2)
         lambda(3)=v(i,2)+axsq
         lambda(4)=v(i,2)-axsq
         lambda(5)=v(i,2)+cfast
         lambda(6)=v(i,2)-cfast
         lambda(7)=v(i,2)+cslow
         lambda(8)=v(i,2)-cslow
c     
c     Eigenvector corresponding to u
         evl(1,1)=1.D0
         evl(1,8)=-1.D0/csndsq
c     
c     Eigenvector corresponding to u
         evl(2,5)=1.D0
c     
c     Eigenvector corresponding to u+ax
         evl(3,3)=-betaZ*TwoSqrtInv
         evl(3,4)=betaY*TwoSqrtInv
         evl(3,6)=1.D0/dsqrt(v(i,1))*TwoSqrtInv*betaZ*SBx
         evl(3,7)=-1.D0/dsqrt(v(i,1))*TwoSqrtInv*betaY*SBx
c     
c     Eigenvector corresponding to u-ax
         evl(4,3)=-betaZ*TwoSqrtInv
         evl(4,4)=betaY*TwoSqrtInv
         evl(4,6)=-evl(3,6)
         evl(4,7)=-evl(3,7)
c     
c     Eigenvector corresponding to u+cfast
         evl(5,1)=0.D0
         evl(5,2)=0.5D0*alphaF*cfast/csndsq
         evl(5,3)=-0.5D0*alphaS*cslow*betaY*Sbx/csndsq
         evl(5,4)=-0.5D0*alphaS*cslow*betaZ*Sbx/csndsq
         evl(5,6)=0.5D0*alphaS/dsqrt(v(i,1)*csndsq)*betaY
         evl(5,7)=0.5D0*alphaS/dsqrt(v(i,1)*csndsq)*betaZ
         evl(5,8)=0.5D0*alphaF/v(i,1)/csndsq
c     
c     Eigenvector corresponding to u-cfast
         evl(6,1)=evl(5,1)
         evl(6,2)=-evl(5,2)
         evl(6,3)=-evl(5,3)
         evl(6,4)=-evl(5,4)
         evl(6,6)=evl(5,6)
         evl(6,7)=evl(5,7)
         evl(6,8)=evl(5,8)
c     
c     Eigenvector corresponding to u+cslow
         evl(7,1)=0.D0
         evl(7,2)=0.5D0*alphaS*cslow/csndsq
         evl(7,3)=0.5D0*alphaF*cfast*betaY*Sbx/csndsq
         evl(7,4)=0.5D0*alphaF*cfast*betaZ*Sbx/csndsq
         evl(7,6)=-0.5D0*alphaF/dsqrt(v(i,1)*csndsq)*betaY
         evl(7,7)=-0.5D0*alphaF/dsqrt(v(i,1)*csndsq)*betaZ
         evl(7,8)=0.5D0*alphaS/v(i,1)/csndsq
c     
c     
c     Eigenvector corresponding to u-cslow
         evl(8,1)=evl(7,1)
         evl(8,2)=-evl(7,2)
         evl(8,3)=-evl(7,3)
         evl(8,4)=-evl(7,4)
         evl(8,6)=evl(7,6)
         evl(8,7)=evl(7,7)
         evl(8,8)=evl(7,8)
c     
c     Right eigenvectors
c     Eigenvector corresponding to u
         evr(1,1)=1.D0
c     
c     Eigenvector corresponding to u
         evr(5,2)=1.D0
c     
c     Eigenvector corresponding to u+ax
         evr(3,3)=-betaZ*TwoSqrtInv
         evr(4,3)=betaY*TwoSqrtInv
         evr(6,3)=dsqrt(v(i,1))*TwoSqrtInv*betaZ*SBx
         evr(7,3)=-dsqrt(v(i,1))*TwoSqrtInv*betaY*SBx
c     
c     Eigenvector corresponding to u-ax
         evr(3,4)=-betaZ*TwoSqrtInv
         evr(4,4)=betaY*TwoSqrtInv
         evr(6,4)=-evr(6,3)
         evr(7,4)=-evr(7,3)
c     
c     Eigenvector corresponding to u+cfast
         evr(1,5)=v(i,1)*alphaF
         evr(2,5)=alphaF*cfast
         evr(3,5)=-alphaS*cslow*betaY*Sbx
         evr(4,5)=-alphaS*cslow*betaZ*Sbx
         evr(6,5)=alphaS*dsqrt(v(i,1)*csndsq)*betaY
         evr(7,5)=alphaS*dsqrt(v(i,1)*csndsq)*betaZ
         evr(8,5)=v(i,1)*alphaF*csndsq
c     
c     Eigenvector corresponding to u-cfast
         evr(1,6)=evr(1,5)
         evr(2,6)=-evr(2,5)
         evr(3,6)=-evr(3,5)
         evr(4,6)=-evr(4,5)
         evr(6,6)=evr(6,5)
         evr(7,6)=evr(7,5)
         evr(8,6)=evr(8,5)
c     
c     Eigenvector corresponding to u+cslow
         evr(1,7)=v(i,1)*alphaS
         evr(2,7)=alphaS*cslow
         evr(3,7)=alphaF*cfast*betaY*Sbx
         evr(4,7)=alphaF*cfast*betaZ*Sbx
         evr(6,7)=-alphaF*dsqrt(v(i,1)*csndsq)*betaY
         evr(7,7)=-alphaF*dsqrt(v(i,1)*csndsq)*betaZ
         evr(8,7)=v(i,1)*alphaS*csndsq
c     
c     Eigenvector corresponding to u-cslow
         evr(1,8)=evr(1,7)
         evr(2,8)=-evr(2,7)
         evr(3,8)=-evr(3,7)
         evr(4,8)=-evr(4,7)
         evr(6,8)=evr(6,7)
         evr(7,8)=evr(7,7)
         evr(8,8)=evr(8,7)
c     
c     Project the slopes to the eigenspace
         do l=1,8,1
            pl(l)=0.D0
            pr(l)=0.D0
            do k=1,8,1
               sum1=0.D0
               do m=1,8,1
                  sum1=sum1+evl(k,m)*dvdx(i,m)
               enddo
               if(lambda(k).gt.0.D0) then
                  pl(l)=pl(l)+sum1*evr(l,k)
               else
                  pr(l)=pr(l)+sum1*evr(l,k)
               endif
            enddo
         enddo
c     
c     Now calculate the plus (left) and minus (right) states
c     
         do l=1,8,1
            pl(l)=-pl(l)*dtodx*0.5D0
            pr(l)=-pr(l)*dtodx*0.5D0
         enddo
         up=-1.D0/dtodx+v(i,2)
         um=1.D0/dtodx+v(i,2)
c     
         vl(i,1)=v(i,1)-pl(1)/dtodx+v(i,2)*pl(1)+v(i,1)*pl(2)
         vl(i,2)=v(i,2)-pl(2)/dtodx+v(i,2)*pl(2)+rhoInv*(v(i,6)*pl(6)+
     &        v(i,7)*pl(7)+pl(8))
         vl(i,3)=v(i,3)-pl(3)/dtodx+v(i,2)*pl(3)-rhoInv*v(i,5)*pl(6)
         vl(i,4)=v(i,4)-pl(4)/dtodx+v(i,2)*pl(4)-rhoInv*v(i,5)*pl(7)
         vl(i,5)=v(i,5)-pl(5)/dtodx+v(i,2)*pl(5)
         vl(i,6)=v(i,6)-pl(6)/dtodx+v(i,2)*pl(6)+v(i,6)*pl(2)
     &        -v(i,5)*pl(3)
         vl(i,7)=v(i,7)-pl(7)/dtodx+v(i,2)*pl(7)+v(i,7)*pl(2)
     &        -v(i,5)*pl(4)
         vl(i,8)=v(i,8)-pl(8)/dtodx+v(i,2)*pl(8)+gamma*v(i,8)*pl(2)
c     
         vr(i,1)=v(i,1)+pr(1)/dtodx+v(i,2)*pr(1)+v(i,1)*pr(2)
         vr(i,2)=v(i,2)+pr(2)/dtodx+v(i,2)*pr(2)+rhoInv*(v(i,6)*pr(6)+
     &        v(i,7)*pr(7)+pr(8))
         vr(i,3)=v(i,3)+pr(3)/dtodx+v(i,2)*pr(3)-rhoInv*v(i,5)*pr(6)
         vr(i,4)=v(i,4)+pr(4)/dtodx+v(i,2)*pr(4)-rhoInv*v(i,5)*pr(7)
         vr(i,5)=v(i,5)+pr(5)/dtodx+v(i,2)*pr(5)
         vr(i,6)=v(i,6)+pr(6)/dtodx+v(i,2)*pr(6)+v(i,6)*pr(2)
     &        -v(i,5)*pr(3)
         vr(i,7)=v(i,7)+pr(7)/dtodx+v(i,2)*pr(7)+v(i,7)*pr(2)
     &        -v(i,5)*pr(4)
         vr(i,8)=v(i,8)+pr(8)/dtodx+v(i,2)*pr(8)+gamma*v(i,8)*pr(2)
c     
      enddo
c     
      return
      end 
c-----------------------------------------------------------------------

