/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _PC_INTERP_H_
#define _PC_INTERP_H_

#include "REAL.H"
#include "LevelData.H"
#include "BaseFab.H"
#include "FArrayBox.H"
#include "ProblemDomain.H"

class DisjointBoxLayout;

/// class to interpolate coarse level data to a fine level
/**
 class to interpolate coarse level data to a fine level 
 data for multigrid.  This class should be considered internal
 to AMRSolver and not a part of the Chombo API.
*/
class MGInterp
{
public:
  ///
  MGInterp();

  ///
  ~MGInterp();

  ///
  MGInterp(const DisjointBoxLayout& a_fine_domain,
           int a_numcomps,
           int a_ref_ratio,
           const Box& a_problem_domain);

  ///
  MGInterp(const DisjointBoxLayout& a_fine_domain,
           int a_numcomps,
           int a_ref_ratio,
           const ProblemDomain& a_problem_domain);

  /// 
  /**
     specify the fine level domain and refinement ratio.
  */
  void 
  define(const DisjointBoxLayout& a_fine_domain, // the fine level domain
         int a_numcomps,                         // the number of components
         int a_ref_ratio,                        // the refinement ratio
         const Box& a_problem_domain);           // problem domain


  /// 
  /**
     specify the fine level domain and refinement ratio.
  */
  void 
  define(const DisjointBoxLayout& a_fine_domain, // the fine level domain
         int a_numcomps,                         // the number of components
         int a_ref_ratio,                        // the refinement ratio
         const ProblemDomain& a_problem_domain); // problem domain

  /// 
  /**
     interpolate coarse level data to fine level    
  */
  void 
  interpToFine(LevelData<FArrayBox>& a_fine_data,
               const LevelData<FArrayBox>& a_coarse_data);

  ///
  bool
  isDefined() const;

protected:
  bool is_defined;
  // the refinement ratio
  int m_ref_ratio;
  // work array for the coarse level data in a domain that is the
  // outline of the fine level domain on the coarse level
  LevelData<FArrayBox> m_coarsened_fine_data;
  DisjointBoxLayout m_grids;
  // problem domain
  ProblemDomain m_problem_domain;
};

#endif
