/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "Box.H"
#include "FArrayBox.H"
#include "REAL.H"
#include "SPACE.H"
#include "Tuple.H"
#include "Vector.H"
#include "LoHiSide.H"

#include "PoissonBC.H"
#include "MayDay.H"

NeumannBC::NeumannBC() : BoxGhostBC()
{
}

NeumannBC::~NeumannBC() 
{
}

void
NeumannBC::fillBCValues(FArrayBox& a_neumfac,
                        FArrayBox& a_dircfac,
                        FArrayBox& a_inhmval,
                        Real a_dx,
                        const Box& a_domain) const
{
  assert(!a_neumfac.box().isEmpty());
  assert(!a_dircfac.box().isEmpty());
  assert(!a_inhmval.box().isEmpty());
  a_neumfac.setVal(1.0);
  a_dircfac.setVal(0.0);
  a_inhmval.setVal(0.0);
}


void
NeumannBC::fillBCValues(FArrayBox& a_neumfac,
                        FArrayBox& a_dircfac,
                        FArrayBox& a_inhmval,
                        Real a_dx,
                        const ProblemDomain& a_domain) const
{
  assert(!a_neumfac.box().isEmpty());
  assert(!a_dircfac.box().isEmpty());
  assert(!a_inhmval.box().isEmpty());
  a_neumfac.setVal(1.0);
  a_dircfac.setVal(0.0);
  a_inhmval.setVal(0.0);
}

void
DircheletBC::fillBCValues(FArrayBox& a_neumfac,
                          FArrayBox& a_dircfac,
                          FArrayBox& a_inhmval,
                          Real a_dx,
                          const Box& a_domain) const
{
  assert(!a_neumfac.box().isEmpty());
  assert(!a_dircfac.box().isEmpty());
  assert(!a_inhmval.box().isEmpty());
  a_neumfac.setVal(0.0);
  a_dircfac.setVal(1.0);
  a_inhmval.setVal(0.0);
}

void
DircheletBC::fillBCValues(FArrayBox& a_neumfac,
                          FArrayBox& a_dircfac,
                          FArrayBox& a_inhmval,
                          Real a_dx,
                          const ProblemDomain& a_domain) const
{
  assert(!a_neumfac.box().isEmpty());
  assert(!a_dircfac.box().isEmpty());
  assert(!a_inhmval.box().isEmpty());
  a_neumfac.setVal(0.0);
  a_dircfac.setVal(1.0);
  a_inhmval.setVal(0.0);
}




BoxGhostBC* 
NeumannBC::new_boxghostbc() const
{
  NeumannBC* newop = new NeumannBC();
  if(newop == NULL)
    {
      MayDay::Error("Out of Memory in NeumannBC::new_boxghostbc");
    }
  return static_cast<BoxGhostBC*>(newop);
}
BoxGhostBC* 
DircheletBC::new_boxghostbc() const
{
  DircheletBC* newop = new DircheletBC();
  if(newop == NULL)
    {
      MayDay::Error("Out of Memory in DircheletBC::new_boxghostbc");
    }
  return static_cast<BoxGhostBC*>(newop);
}


NeumannBC::NeumannBC(int dir, Side::LoHiSide side, const Interval& a_comps)
    : BoxGhostBC(dir,side,a_comps)
{
}


NeumannBC::NeumannBC(int dir, Side::LoHiSide side)
  : BoxGhostBC(dir,side)
{
}

DircheletBC::DircheletBC() : BoxGhostBC()
{
}

DircheletBC::~DircheletBC()
{
}

DircheletBC::DircheletBC(int dir, Side::LoHiSide side)
    : BoxGhostBC(dir,side)
{
}

DircheletBC::DircheletBC(int dir, Side::LoHiSide side, const Interval& a_comps)
    : BoxGhostBC(dir,side,a_comps)
{
}


