/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cstring>
#include "BitSet.H"
#include <cstdlib>
#include <malloc.h>
#include "MayDay.H"

BitSet BitSetIterator::emptyBitSet = BitSet();

WORD BitSet::trueMasks[WORDSIZE];

int BitSet::initialize()
{
  trueMasks[WORDSIZE-1] = 1;
  for(int i=WORDSIZE-2; i>=0 ; --i)
    {
      trueMasks[i] = trueMasks[i+1] << 1;
    }
  return 0;
}

BitSet::BitSet()
  :
  m_bits(NULL),
  m_size(0),
  m_length(0)
{}

long int BitSet::bytes = BitSet::initialize();
long int BitSet::peak = 0;

BitSet::BitSet(int bits, bool init)
  :
  m_bits(NULL),
  m_size(bits),
  m_length(0)
{
  //  static int initialized = initialize();
  assert(bits >= 0);
  if(bits==0) return;

  m_length = bits/WORDSIZE;
  if(m_length*WORDSIZE != bits) m_length +=1;

  m_bits = (WORD*)malloc(m_length*sizeof(WORD));
  bytes+= m_length*sizeof(WORD);
  if(bytes > peak) peak = bytes;

  if(m_bits == NULL)
    {
      MayDay::Error("Memory Error in BitSet::BitSet(int bits, bool init)");
    }
  if(init)
    {
      memset(m_bits, ~0, m_length*sizeof(WORD));
    }
  else
    {
      memset(m_bits, 0, m_length*sizeof(WORD));
    }
}

void BitSet::define(int bits, bool init)
{
  if(m_bits != NULL){
    free(m_bits);
    bytes-=m_length*sizeof(WORD);
  }
  m_size = bits;
  assert(bits >= 0);
  m_length = bits/WORDSIZE;
  if(bits == 0) { m_bits = NULL; return;}
  if(m_length*WORDSIZE != bits) m_length +=1;
  m_bits = (WORD*)malloc(m_length*sizeof(WORD));
  bytes+= m_length*sizeof(WORD);
  if(bytes > peak) peak = bytes;

  if(m_bits == NULL)
    {
      MayDay::Error("Memory Error in BitSet::BitSet(int bits, bool init)");
    }
  if(init)
    {
      memset(m_bits, ~0, m_length*sizeof(WORD));
    }
  else
    {
      memset(m_bits, 0, m_length*sizeof(WORD));
    }
}

BitSet::BitSet(const BitSet& rhs)
  :
  m_bits(NULL),
  m_size(rhs.m_size),
  m_length(rhs.m_length)
{
  if(m_length == 0) return;
  m_bits = (WORD*)malloc(m_length*sizeof(WORD));
  bytes+= m_length*sizeof(WORD);
  if(bytes > peak) peak = bytes;
  if(m_bits == NULL)
    {
      MayDay::Error("Memory Error in BitSet::BitSet(const BitSet& rhs)");
    }

  memcpy(m_bits, rhs.m_bits, m_length*sizeof(WORD));
}

bool BitSet::isEmpty() const
{
  if(m_length == 0) return true;
  for(int i=0; i<m_length-1; ++i)
    {
      if(m_bits[i] > 0) return false;
    }
  for(int i=(m_length-1)*WORDSIZE; i<m_size; ++i)
    {
      if (this->operator[](i)) return false;
    }
  return true;
}

bool BitSet::isFull() const
{
  WORD g=0;
  g=~g;
  for(int i=0; i<m_length-1; ++i)
    {
      if(m_bits[i] != g) return false;
    }
  for(int i=(m_length-1)*WORDSIZE; i<m_size; ++i)
    {
      if (!this->operator[](i)) return false;
    }
  return true;
}

BitSet& BitSet::operator=(const BitSet& rhs)
{
  if(m_bits != NULL) 
    {
      free(m_bits);
      bytes -= m_length*sizeof(WORD);
      m_bits = NULL;
    }
  m_length=rhs.m_length;
  m_size =rhs.m_size;

  if(rhs.m_bits != NULL)
    {
      m_bits = (WORD*)malloc(m_length*sizeof(WORD));
      bytes+= m_length*sizeof(WORD);
      if(bytes > peak) peak = bytes;
    }
  else
    {
      m_bits = NULL;
      return *this;
    }
  if(m_bits == NULL)
    {
      MayDay::Error("Memory Error in BitSet::operator=(const BitSet& rhs)");
    }

  memcpy(m_bits, rhs.m_bits, m_length*sizeof(WORD));
  return *this;
}

BitSet::~BitSet()
{
  if(m_bits != NULL) 
    {
      free(m_bits);
      bytes-=m_length*sizeof(WORD);
    }
}

void BitSet::setFalse(int i)
{
  // WORD* tm = trueMasks;
  assert(i>=0);
  assert(i<m_size);
  int index = i/WORDSIZE;
  assert(index < m_length);
  int remainder = i-WORDSIZE*index;
  WORD* word = m_bits+index;

  *word = *word & ~(trueMasks[remainder]);
}

void BitSet::setTrue(int i)
{
  assert(i>=0);
  assert(i<m_size);
  int index = i/WORDSIZE;
  assert(index < m_length);
  int remainder = i-WORDSIZE*index;
  WORD* word = m_bits+index;

  *word = *word | trueMasks[remainder];
}


void BitSetIterator::end()
{
  m_index = m_length;
  m_remainder = m_partialBits;
}
void BitSetIterator::begin()
{
  m_index = 0;
  m_remainder = 0;
}
