/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _PARMPARSE_IO_H_
#define _PARMPARSE_IO_H_
#include <iostream>
#include <fstream>
using std::ofstream;
#include <string>

#include "REAL.H" 
class Box;
class IntVect;
class FArrayBox;
class BoxLayout;
class DisjointBoxLayout;
template <class TTYPE> class LevelData;

class ParmParse;

class ParmParseIO
{
public:
  ParmParseIO ();
  ~ParmParseIO ();

  void
  openForRead (const std::string& a_filename);

  void
  openForWrite (const std::string& a_filename);

  void
  close ();

  void
  setGroup (const std::string& a_groupname);

  const std::string
  getGroup () const;

    const std::string&
    filename() const;

    const std::string&
    groupname() const;

  void set (const std::string& a_name, const Real&        a_value);
  void set (const std::string& a_name, const int&         a_value);
  void set (const std::string& a_name, const std::string& a_value);
  void set (const std::string& a_name, const IntVect&     a_value);
  void set (const std::string& a_name, const Box&         a_value);

  void get (const std::string& a_name, Real&        a_value) const;
  void get (const std::string& a_name, int&         a_value) const;
  void get (const std::string& a_name, std::string& a_value) const;
  void get (const std::string& a_name, IntVect&     a_value) const;
  void get (const std::string& a_name, Box&         a_value) const;

  bool
  found (const std::string& a_name) const;

  friend
  void
  write (ParmParseIO& a_ppio, const BoxLayout& a_layout);

// return code:
// 0: successful read
// 1: no num_boxes
// 2: no box_n (does not report n)
  friend
  int
  read (ParmParseIO& a_ppio, Vector<Box>& a_boxes);

// return codes:
// 0: successful read
// 1: no num_comps
// 2: bad fab read

protected:
  ParmParse* m_parmparse_ptr;
  ofstream m_ostrm;
  std::string m_groupname;
  std::string m_filename;
};


  void
  write (ParmParseIO& a_ppio, 
         const LevelData<FArrayBox>& a_data, 
         const std::string& a_name);

int
read  (ParmParseIO& a_ppio, 
                  LevelData<FArrayBox>& a_data, 
                  const std::string& a_name,
                  const DisjointBoxLayout& a_layout);



#endif
